#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

from pyramid.decorator import reify
from z3c.form.browser.widget import HTMLInputWidget
from z3c.form.converter import BaseDataConverter
from z3c.form.interfaces import IDataConverter, IErrorViewSnippet, IFieldWidget, IFormAware, IValidator, NO_VALUE
from z3c.form.util import expandPrefix
from z3c.form.widget import FieldWidget, Widget
from zope.interface import alsoProvides, implementer_only
from zope.schema import ValidationError

# import interfaces
from pyams_form.interfaces.form import IFormLayer
# import packages
from pyams_form.widget import SEOTextLineFieldWidget, widgettemplate_config
from pyams_i18n.interfaces import II18nManager, INegotiator
from pyams_i18n.interfaces.schema import II18nField, II18nFileField, II18nHTMLField, II18nTextField, II18nTextLineField
from pyams_i18n.interfaces.widget import II18nFileWidget, II18nHTMLWidget, II18nSEOTextLineWidget, II18nTextLineWidget, \
    II18nTextWidget, II18nWidget
from pyams_utils.adapter import adapter_config
from pyams_utils.registry import query_utility
from pyams_utils.traversing import get_parent


@adapter_config(context=(II18nField, II18nWidget), provides=IDataConverter)
class I18nDataConverter(BaseDataConverter):
    """I18n base data converter"""

    def __init__(self, field, widget):
        super(I18nDataConverter, self).__init__(field, widget)

    def toWidgetValue(self, value):
        # I18n widget is using a dict where each key is a lang
        # and each value is matching widget value
        result = {}
        if value is None:
            return result
        registry = self.widget.request.registry
        for lang, val in value.items():
            converter = registry.queryMultiAdapter((self.field.value_type, self.widget.get_widget(lang)),
                                                   IDataConverter)
            if converter is not None:
                result[lang] = converter.toWidgetValue(val)
        return result

    def toFieldValue(self, value):
        # I18n widget is using a dict where each key is a lang
        # and each value is matching widget value
        result = {}
        registry = self.widget.request.registry
        for lang in self.widget.langs:
            converter = registry.queryMultiAdapter((self.field.value_type, self.widget.get_widget(lang)),
                                                   IDataConverter)
            if converter is not None:
                result[lang] = converter.toFieldValue(self.widget.get_value(lang))
        return result


@widgettemplate_config(mode='input', template='templates/i18n-input.pt', widget=II18nWidget, layer=IFormLayer)
@widgettemplate_config(mode='display', template='templates/i18n-input.pt', widget=II18nWidget, layer=IFormLayer)
@widgettemplate_config(mode='hidden', template='templates/i18n-hidden.pt', widget=II18nWidget, layer=IFormLayer)
@implementer_only(II18nWidget)
class I18nWidget(HTMLInputWidget, Widget):
    """I18n base widget"""

    widgets = None
    widget_factory = None

    @reify
    def langs(self):
        langs = []
        negotiator = query_utility(INegotiator)
        if negotiator is not None:
            langs.append(negotiator.server_language)
        manager = get_parent(self.context, II18nManager)
        if manager is not None:
            langs.extend(sorted(filter(lambda x: x not in langs, manager.languages or ())))
        elif negotiator is not None:
            langs.extend(sorted(filter(lambda x: x not in langs, negotiator.offered_languages or ())))
        else:
            langs.append('en')
        return langs

    def update(self):
        registry = self.request.registry
        self.widgets = {}
        for lang in self.langs:
            if self.widget_factory is not None:
                widget = self.widget_factory(self.field.value_type, self.request)
            else:
                widget = registry.queryMultiAdapter((self.field.value_type, self.request), IFieldWidget)
            if widget is not None:
                base_prefix = expandPrefix(self.form.prefix) + expandPrefix(self.form.widgets.prefix)
                prefix = base_prefix + expandPrefix(lang)
                name = self.field.value_type.__name__ = self.field.__name__
                widget.mode = self.mode
                widget.basename = base_prefix + name
                widget.name = prefix + name
                widget.id = widget.name.replace('.', '-')
                if IFormAware.providedBy(self):
                    widget.form = self.form
                    alsoProvides(widget, IFormAware)
                widget.field = self.field.value_type
                widget.context = self.context
                widget.ignoreContext = self.ignoreContext
                widget.ignoreRequest = self.ignoreRequest
                widget.lang = lang
                widget.update()
            self.widgets[lang] = widget
        super(I18nWidget, self).update()
        for lang in self.langs:
            widget = self.widgets[lang]
            value = (self.value or {}).get(lang, NO_VALUE)
            if value is not NO_VALUE:
                try:
                    converter = IDataConverter(widget)
                    field_value = converter.toFieldValue(value)
                    registry.getMultiAdapter((self.context, self.request, self.form, widget.field, widget),
                                             IValidator).validate(field_value)
                    widget.value = converter.toWidgetValue(field_value)
                except (ValidationError, ValueError) as error:
                    view = registry.getMultiAdapter((error, self.request, widget, widget.field, self.form,
                                                     self.context), IErrorViewSnippet)
                    view.update()
                    widget.error = view
                    widget.value = value

    def set_mode(self, mode):
        if self.widgets:
            for lang in self.langs:
                self.widgets[lang].mode = mode
        self.mode = mode

    def extract(self, default=NO_VALUE):
        result = {}
        [result.setdefault(lang, self.widgets[lang].extract(default)) for lang in self.langs]
        for value in result.values():
            if value is not NO_VALUE:
                return result
        return NO_VALUE
    
    def get_widget(self, lang):
        return self.widgets.get(lang)

    def get_value(self, lang):
        return self.get_widget(lang).value


@implementer_only(II18nTextLineWidget)
class I18nTextLineWidget(I18nWidget):
    """I18n text line widget"""


@adapter_config(context=(II18nTextLineField, IFormLayer), provides=IFieldWidget)
def I18nTextLineFieldWidget(field, request):
    """I18n text line field widget factory"""
    return FieldWidget(field, I18nTextLineWidget(request))


@implementer_only(II18nSEOTextLineWidget)
class I18nSEOTextLineWidget(I18nTextLineWidget):
    """I18n text line widget with SEO quality marker"""


def I18nSEOTextLineFieldWidget(field, request):
    """I18n text line field widget with SEO quality marker factory"""
    widget = I18nSEOTextLineWidget(request)
    widget.widget_factory = SEOTextLineFieldWidget
    return FieldWidget(field, widget)


@implementer_only(II18nTextWidget)
class I18nTextWidget(I18nWidget):
    """I18n text widget"""


@adapter_config(context=(II18nTextField, IFormLayer), provides=IFieldWidget)
def I18nTextFieldWidget(field, request):
    """I18n text field widget factory"""
    return FieldWidget(field, I18nTextWidget(request))


@implementer_only(II18nHTMLWidget)
class I18nHTMLWidget(I18nWidget):
    """I18n HTML widget"""


@adapter_config(context=(II18nHTMLField, IFormLayer), provides=IFieldWidget)
def I18nHTMLFieldWidget(field, request):
    """I18n HTML field widget factory"""
    return FieldWidget(field, I18nHTMLWidget(request))


@implementer_only(II18nFileWidget)
class I18nFileWidget(I18nWidget):
    """I18n file widget"""


@adapter_config(context=(II18nFileField, IFormLayer), provides=IFieldWidget)
def I18nFileFieldWidget(field, request):
    """I18n file field widget factory"""
    return FieldWidget(field, I18nFileWidget(request))
